##############################################################################
##############################################################################
##  
##  PostgreSQL install script for AerOn, SmartGpio
##
##  Author:   John Felix
##  Date:     10-04-2025
##  Company:  Broadcast Partners
##
##############################################################################
##
##  Website:    http://www.aeronstudio.nl
##  Helpdesk:   https://support.broadcastpartners.nl
##  Solutions:  https://support.broadcastpartners.nl/support/solutions
##  Info:       https://www.broadcastpartners.nl/aeron
##
###  Share feedback, suggestions and ideas to support@broadcastpartners.nl
##
##############################################################################
##
##  This script supports PowerShell 3.0 and up
##
##  The latest script version can be downloaded from
##  http://update.aeronstudio.nl/files/release/install-postgresql-aeron.zip
##
##############################################################################
##
###  Script parameters are Xplained in Install-Postgresql-Aeron-ReadMe.pdf
##
##  You should read and consult Install-Postgresql-Aeron-ReadMe.pdf before installation.
##
##  Online manual: https://support.broadcastpartners.nl/support/solutions/articles/52000081691-Aeron-Installatie-PostgreSQL-Aeron-Studio
##
##############################################################################
##
### Script execution policies
##
## Current policy: get-executionPolicy
##
## Allow execution: set-executionpolicy Bypass
## Prevent execution:  set-executionpolicy -ExecutionPolicy Restricted
##
## Allow remote: set-executionpolicy Remotesigned; 
##
##############################################################################
##
## PostgreSQL major version 12 is officially supported from AerOn v1.13.4.22
## PostgreSQL major version 13 is officially supported from AerOn v1.14.9.13
## PostgreSQL major version 14 is officially supported from AerOn v1.15.10.28
## PostgreSQL major version 15 is officially supported from AerOn v1.16.3.17
## Advised: PostgreSQL major version 16 is officially supported from AerOn v1.16.11.14
##
### PostgreSQL major version 17 is not yet oficially supported, but seems to work well. You can use version 17 for testing.
##
##############################################################################
##############################################################################

# REGION Parameters

param(
[string]$pgVersion = '16',
[string]$pgInstallDir ='C:\Program Files\PostgreSQL\' + $pgVersion, 
[string]$pgDataDir ='C:\Aeron Database\PostgreSQL\' + $pgVersion + '\Database',
[string]$pgBackupDir ='C:\Aeron Database\PostgreSQL\' + $pgVersion + '\Backup',
[string]$pgHostname = '127.0.0.1', 
[string]$pgPort = '54' + $pgVersion, 
[string]$pgDatabase = 'aeron_prod_db',
[string]$pgDBschema = 'public',
[string]$pgSuperUser = 'postgres',
[string]$pwdSuperUser = 'postgres',
[String]$AeronDBadmin = 'aeron_dba',
[string]$pwdAeronDBadmin = 'aeron_dba',
[string]$AeronAppUser = 'aeron_vt_user',
[string]$pwdAeronAppUser = 'aeron_vt_user', 
[string]$pgServiceName = 'postgresql-' + $pgVersion + '-x64-aeron',  
[string]$pgServiceUser = 'postgres', 
[string]$pgServicePassword = 'postgres',

[bool]$CreatePgServiceUser=$false,        # $CreateLocalUser=$true   Needed for setup PostgreSQL Master/Slave streaming replication
[bool]$OnlyDownload=$false,              # $OnlyDownload=$true      Just download files, skip installation
[bool]$RunDbConvert=$false);             # $RunDbConvert=$true      Run NexusDB database to PostgreSQL database conversion program

# END REGION Parameters


$ScriptPath = $PSScriptRoot; # supported from PowerShell 3.0
#$ScriptPath = split-path -parent $MyInvocation.MyCommand.Definition


# REGION Postgresql settings

if ($CreatePgServiceUser) {
  [string]$pgServiceAccount=$pgServiceUser;
} else {
  [string]$pgServiceAccount="NT AUTHORITY\NETWORKSERVICE";
  $pgServicePassword="";
}

$pgBinDir = "$pgInstallDir\bin";

# Configuration files
$postgresqlConf = "$pgDataDir\postgresql.conf";
$pg_hbaConf     = "$pgDataDir\pg_hba.conf";

# END REGION Postgresql settings


# REGION Installer declarations

$Is32bit = $env:PROCESSOR_ARCHITECTURE -eq 'x86';
$Is64bit = $env:PROCESSOR_ARCHITECTURE -eq 'AMD64';

$dirAuto          = "$ScriptPath\auto";
$dirSql           = "$dirAuto\sql";
$dirConf          = "$dirAuto\conf";
$dirConfTrust     = "$dirConf\trust";
$dirCmd           = "$dirAuto\cmd";
$dirScripts       = "$ScriptPath\scripts";
$dirScriptsPsql   = "$dirScripts\psql";

$fileAeronConf      = "$dirConf\aeron.conf";
$filePgHbaConf      = "$dirConf\pg_hba.conf";
$filePgHbaTrustConf = "$dirConfTrust\pg_hba.conf";

$batchCreateUsersDatabase       = "$dirAuto\create_users_" + $pgDatabase + ".bat";
$batchGrantTablesToAeronAppRole = "$dirAuto\grant_tables_to_aeron_app_role.bat";
$batchRetartPostgresql          = "$dirAuto\restart-postgresql.bat";

$batchAddFirewallRulePostgresql = "$dirCmd\add_firewall_rule_postgresql.bat";
$batchCopyPostgresqlConf        = "$dirCmd\copy_postgresql_conf.bat";
$batchCopyPostgresqlHba         = "$dirCmd\copy_postgresql_Hba.bat";

$sqlCreateUsers                 = "$dirSql\1-create-users.sql";
$sqlCreateDatabase              = "$dirSql\2-create-" + $pgDatabase + ".sql";
$sqlSetConnectOptions           = "$dirSql\3-set-connect-options-" + $pgDatabase + ".sql";
$sqlCreateSchema                = "$dirSql\4-create-schema-" + $pgDatabase + ".sql";
$sqlGrantTablesToAeronAppRole   = "$dirSql\5-grant-tables-to-aeron-app-role.sql";
$sqlSetPasswords                = "$dirSql\set-passwords.sql";

$scriptReloadPsqlConfiguration  = "$dirScriptsPsql\reload_postgresql_conf.bat";
$scriptBackupDatabase           = "$dirScriptsPsql\backup_database.bat";
$scriptRestoreDatabase          = "$dirScriptsPsql\restore_database.bat";
$scriptCreateDatabase           = "$dirScriptsPsql\create_database.bat";

$filePgInstallerOption = "$ScriptPath\postgresql_install.opt";
$fileInstallSteps      = "$ScriptPath\steps-to-install.txt";

if ($Is64bit) {
  Write-Host "System is 64-bit Windows";
  Write-Host "PostgreSQL version selected is "$pgVersion;
  # The latest PostgreSQL builds are available from 
  #    https://www.enterprisedb.com/downloads/postgres-postgresql-downloads
  # Release notes:  https://www.postgresql.org/docs/release
  # Documentation:  https://www.postgresql.org/docs
  if ($pgVersion -eq '15') {
    $pgInstallerUrl  = "http://update.aeronstudio.nl/files/postgres/postgresql-15.exe";
    $pgInstallerPath = "$ScriptPath\postgresql-x64-15.12-1.exe";
    $pgInstallerLnk  = "$ScriptPath\postgresql-x64-15.12-1.lnk";
  } elseif ($pgVersion -eq '16') {
    $pgInstallerUrl  = "http://update.aeronstudio.nl/files/postgres/postgresql-16.exe";
    $pgInstallerPath = "$ScriptPath\postgresql-x64-16.8-1.exe";
    $pgInstallerLnk  = "$ScriptPath\postgresql-x64-16.8-1.lnk";
  } elseif ($pgVersion -eq '17') {
    $pgInstallerUrl  = "http://update.aeronstudio.nl/files/postgres/postgresql-17.exe";
    $pgInstallerPath = "$ScriptPath\postgresql-x64-17.4-1.exe";
    $pgInstallerLnk  = "$ScriptPath\postgresql-x64-17.4-1.lnk";	
  } else { # If entered pgVersion is not supported by script, install version 16
	$pgInstallerUrl  = "http://update.aeronstudio.nl/files/postgres/postgresql-16.exe";
    $pgInstallerPath = "$ScriptPath\postgresql-x64-16.8-1.exe";
    $pgInstallerLnk  = "$ScriptPath\postgresql-x64-16.8-1.lnk";
    $pgVersion = '16';
    $pgInstallDir ='C:\Program Files\PostgreSQL\' + $pgVersion; 
    $pgDataDir ='C:\Aeron Database\PostgreSQL\' + $pgVersion + '\Database';
    $pgBackupDir ='C:\Aeron Database\PostgreSQL\' + $pgVersion + '\Backup';
    $pgPort = '54' + $pgVersion; 
    $pgServiceName = 'postgresql-' + $pgVersion + '-x64-aeron'; 
    $pgBinDir = "$pgInstallDir\bin";
    $postgresqlConf = "$pgDataDir\postgresql.conf";
    $pg_hbaConf     = "$pgDataDir\pg_hba.conf";	
  }	  
} else {
  Write-Host "32-bit Windows not compatible";
  Write-Host "Press any key to exit.
  ";
  [void][System.Console]::ReadKey($true);
  
  exit;
}

$pgAdmin4Path = "$pgInstallDir\pgAdmin 4\bin\pgAdmin4.exe";
$pgAdmin4Dir  = "$pgInstallDir\pgAdmin 4\bin";
$pgAdmin4Lnk  = "$ScriptPath\pgAdmin 4.lnk";

$aeronInstallerUrl  = "http://update.aeronstudio.nl/files/release/SetupAeron2.0.3.28.exe";
$aeronInstallerPath = "$ScriptPath\SetupAeron2.0.3.28.exe";

$aeronHandleidingUrl  = "http://update.aeronstudio.nl/doc/aeron_studio_handleiding.pdf";
$aeronHandleidingPath = "$ScriptPath\AerOn_Studio_Handleiding.pdf";

$aeronManualUrl       = "http://update.aeronstudio.nl/doc/aeron_studio_manual.pdf";
$aeronManualPath      = "$ScriptPath\AerOn_Studio_Manual.pdf";

$aeronSoftwarePath    = "C:\Program Files (x86)\Broadcast Partners\AeronPostgres";
$aeronDbConvertUrl    = "http://update.aeronstudio.nl/files/release/NexusDbToPostgres.exe";
$aeronDbConvertPath   = "$ScriptPath\NexusDbToPostgres.exe";

$name_copy_postgresql_conf           = 'copy_postgresql_conf';
$lnkCopyPostgresqlConf               = "$ScriptPath\$name_copy_postgresql_conf.lnk";
$name_copy_postgresql_Hba            = 'copy_postgresql_Hba';
$lnkCopyPostgresqlHba                = "$ScriptPath\$name_copy_postgresql_Hba.lnk";
$name_add_firewall_rule_postgresql   = 'add_firewall_rule_postgresql';
$lnkAddFirewallRulePostgresql        = "$ScriptPath\$name_add_firewall_rule_postgresql.lnk";
$name_restart_postgresql             = 'restart_postgresql';
$lnkRestartPostgresql                = "$ScriptPath\$name_restart_postgresql.lnk";
$name_reload_postgresql_conf         = 'reload_postgresql_conf';
$lnkReloadPsqlConfiguration          = "$ScriptPath\$name_reload_postgresql_conf.lnk";
$name_grant_tables_to_aeron_app_role = 'grant_tables_to_aeron_app_role';
$lnkGrantTablesToAeronAppRole        = "$ScriptPath\$grant_tables_to_aeron_app_role.lnk";

# END REGION Installer declarations


# REGION Functions
function pgoptionfile
{
# postgresql_install.opt
$Content = "mode=unattended
enable-components=server,commandlinetools,pgAdmin,stackbuilder
prefix=" + $pgInstallDir + "
datadir=" + $pgDataDir + "
superaccount=" + $pgSuperUser + "
superpassword=" + $pwdSuperUser + "
servicename=" + $pgServiceName + "
serviceaccount=" + $pgServiceAccount + "
servicepassword="  + $pgServicePassword + "
serverport=" + $pgPort;
$Content | Set-Content -Path $filePgInstallerOption -Force;
}

function aeronconf
{
# aeron.conf
$Content = "# PostgreSQL configuration override addon file for AerOn.

# To load aeron.conf, add this line at the bottom of postgresql.conf:  include_if_exists = 'aeron.conf'

# The settings and values below will then override the default values defined in postgresql.conf.


# - Connection settings
listen_addresses = '*'
port = " + $pgPort + "
max_connections = 100   # Each AerOn client uses about 30 connections

# - Memory settings
shared_buffers = 512MB  # max 25% available memory. For Windows systems, 512MB is the best value in most cases.
temp_buffers = 80MB
work_mem = 8MB           # max (20% available memory / max_connections)    
maintenance_work_mem = 128MB # max 5% available memory, max 1GB
effective_cache_size = 2GB   # max 50% available memory

# - Background writer settings
bgwriter_delay = 100ms
bgwriter_lru_maxpages = 200     # 0-1000 max buffers written/round

# - Parallel query
max_worker_processes = 12            # Set max to 2x number of real CPU cores on a PC bare metal. On a VM set max to number of available CPU cores.
max_parallel_workers_per_gather = 0  # For AerOn best value is 0. Set max to number of available CPU cores devided by 2. 
max_parallel_workers = 6             # Set max to number of CPU cores.

# - for replication (set default to off)
synchronous_commit = off 	
wal_level = minimal         # wal_level = replica
archive_mode = off          # archive_mode = on
archive_command = ''		# archive_command = 'copy " + '"%p" ' +  '"' + 'P:\\PostgreSQL\\10\\Archive\\%f"' + "'" + "
max_wal_senders = 0         # max_wal_senders = 10
max_replication_slots = 0   # max_replication_slots = 10
hot_standby = off           # hot_standby = on

# - Checkpoint settings
checkpoint_timeout = 10min
checkpoint_completion_target = 0.9
min_wal_size = 80MB
max_wal_size = 4GB

# - Use of SSD.
# random_page_cost = 1.1 # Default value = 4. If database is on SSD uncomment and set value 1.1 else if database is on HDD set value 4.

# - Logging
log_destination = 'stderr'
logging_collector = on
log_directory = 'log'
log_filename = 'postgresql-%a.log'
log_truncate_on_rotation = on
log_rotation_age = 1d
log_rotation_size = 100MB
log_min_duration_statement = 10000
log_checkpoints = on
log_connections = on
log_disconnections = on
log_line_prefix = '%t [%p]: [%l-1] %d %u %a %h '
log_lock_waits = on
log_temp_files = 0

# - Locale and formatting
client_encoding = UTF8

# - TCP Keepalives
tcp_keepalives_idle = 60		
tcp_keepalives_interval = 60		
tcp_keepalives_count = 10

# - Autovacuum and Analyze
autovacuum_vacuum_threshold = 40 
autovacuum_vacuum_scale_factor = 0.01  
      
autovacuum_analyze_threshold = 40
autovacuum_analyze_scale_factor = 0.01 

# Postgres 13 and up
autovacuum_vacuum_insert_threshold = 500
autovacuum_vacuum_insert_scale_factor = 0.01";
$Content | Set-Content -Path $fileAeronConf -Force;
}

function pghbaconf
{
# pg_hba_trust.conf
$Content = "# PostgreSQL client authentication file
# Documentation: https://www.postgresql.org/docs/10/static/auth-pg-hba-conf.html

# Install PostgreSQL database in a location where no user can access this pg_hba authentication file


# TYPE    DATABASE       USER            ADDRESS                METHOD

host    all     " + $pgSuperUser + "             127.0.0.1/32   trust  # trust = Authentication without password
host    all     " + $AeronDBadmin + "             127.0.0.1/32   trust
host    " + $pgDatabase + "   " + $AeronDBadmin + "             127.0.0.1/32   trust

# IPv4 local connections:

# md5 for encrypted password protection
host    all     " + $pgSuperUser + "             127.0.0.1/32   md5
# host	all		" + $pgSuperUser + "			0.0.0.0/0	md5
host    " + $pgDatabase + "   " + $AeronDBadmin + " 	0.0.0.0/0        md5
host    " + $pgDatabase + "   " + $AeronAppUser + "  	0.0.0.0/0        md5
host    smartgpio   smartgpio  	 0.0.0.0/0        md5
host    visualradio   visualradio  	 0.0.0.0/0        md5

# IPv6 local connections:
host    all             all             ::1/128                 md5

# Allow replication connections from localhost, by a user with the replication privilege.
host    replication     all             127.0.0.1/32            md5
host    replication     all             ::1/128                 md5";
$Content | Set-Content -Path $filePgHbaTrustConf -Force;


# pg_hba.conf
$Content = "# PostgreSQL client authentication file
# Documentation: https://www.postgresql.org/docs/10/static/auth-pg-hba-conf.html

# Install PostgreSQL database in a location where no user can access this pg_hba authentication file
# comment trust access rules for superuser " + $pgSuperUser + " when system setup is completed.

# TYPE    DATABASE       USER            ADDRESS                METHOD

# host    all     " + $pgSuperUser + "             127.0.0.1/32   trust     # trust = Authentication without password
# host    all     " + $AeronDBadmin + "             127.0.0.1/32   trust    # all = Grant access to all databases

host    " + $pgDatabase + "   " + $AeronDBadmin + "             127.0.0.1/32   trust 

# IPv4 local connections:

# use md5 for encrypted password protection
host    all     " + $pgSuperUser + "             127.0.0.1/32   md5  # Grant access from localhost
# host	all		" + $pgSuperUser + "			0.0.0.0/0	md5      # Grant access from every ip-address

host    all   " + $AeronDBadmin + " 	0.0.0.0/0        md5
host    all   " + $AeronAppUser + "  	0.0.0.0/0        md5
host    smartgpio   smartgpio  	 0.0.0.0/0        md5
host    visualradio   visualradio  	 0.0.0.0/0        md5

# IPv6 local connections:
host    all             all             ::1/128                 md5

# Allow replication connections from localhost, by a user with the replication privilege.
host    replication     all             127.0.0.1/32            md5
host    replication     all             ::1/128                 md5";
$Content | Set-Content -Path $filePgHbaConf -Force;
}


function sqlfiles
{
# create file 1-create-users.sql
$Content = "-- run as superuser " + $pgSuperUser + "
CREATE USER " + $AeronDBadmin + " WITH PASSWORD '" + $pwdAeronDBadmin + "';
CREATE USER " + $AeronAppUser + " WITH PASSWORD '" + $pwdAeronAppUser + "'; 
CREATE USER smartgpio WITH PASSWORD 'smartgpio';
CREATE USER visualradio WITH PASSWORD 'visualradio';
CREATE ROLE aeron_app_role;
GRANT aeron_app_role TO " + $AeronAppUser + ";
ALTER USER " + $AeronDBadmin + " WITH CREATEDB REPLICATION;
ALTER USER smartgpio WITH CREATEDB REPLICATION;
ALTER USER visualradio WITH CREATEDB;
COMMENT ON ROLE " + $AeronDBadmin + " IS 'AerOn database Owner ';
COMMENT ON ROLE smartgpio IS 'smartgpio database Owner ';
COMMENT ON ROLE " + $AeronAppUser + " IS 'AerOn user Remote VoiceTracking ';"
$Content | Set-Content -Path $sqlCreateUsers -Force;

# create file 2-create-aeron_prod_db.sql
$Content = "-- run as superuser " + $pgSuperUser + "
CREATE DATABASE " + $pgDatabase + " ENCODING UTF8 OWNER " + $AeronDBadmin + ";" + "
CREATE DATABASE smartgpio ENCODING UTF8 OWNER smartgpio;"
$Content | Set-Content -Path $sqlCreateDatabase -Force;

# create file 3-set-connect-options-aeron_prod_db.sql
$Content = "-- run as user " + $AeronDBadmin + "
GRANT CONNECT ON DATABASE " + $pgDatabase + " TO aeron_app_role;";
$Content | Set-Content -Path $sqlSetConnectOptions -Force;

# create file 4-create-schema-aeron_prod_db.sql
$Content = '-- run as user ' + $AeronDBadmin + '
REVOKE CONNECT ON DATABASE ' + $pgDatabase + ' FROM public;
CREATE SCHEMA ' + $pgDBschema + ' AUTHORIZATION ' + $AeronDBadmin + ';
ALTER DATABASE ' + $pgDatabase + ' SET search_path = "$user", ' + $pgDBschema + ', public;';
$Content | Set-Content -Path $sqlCreateSchema -Force;

# create file 5-grant-tables-to-aeron-app-role.sql
$Content = "-- run as user " + $AeronDBadmin + "
GRANT USAGE ON SCHEMA $pgDBschema TO aeron_app_role;
GRANT ALL ON ALL TABLES IN SCHEMA $pgDBschema TO aeron_app_role;";
$Content | Set-Content -Path $sqlGrantTablesToAeronAppRole -Force;

# create file set-passwords.sql
$Content = " -- run as superuser " + $pgSuperUser + "
ALTER USER " + $pgSuperUser + " WITH PASSWORD '" + $pwdSuperUser + "';
ALTER USER " + $AeronDBadmin + " WITH PASSWORD '" + $AeronDBadmin + "';
ALTER USER " + $AeronAppUser + " WITH PASSWORD '" + $AeronAppUser + "';";
$Content | Set-Content -Path $sqlSetPasswords -Force;
}


function installerscripts
{
# copy_postgresql_conf.bat, run by installer
$Content = '@ECHO OFF

REM Script: Copy PostgreSQL config files
REM Author: John Felix
REM Helpdesk: https://support.broadcastpartners.nl/support/home 
REM Share feedback, suggestions and ideas to support@broadcastpartners.nl

REM Automated installation - Run this script as Administrator

Echo.
Echo STEP 2, 3, 4
Echo.
Echo Copy configuration files aeron.conf pg_hba.conf trusted
Echo.

Ren "' + $pg_hbaConf + '" pg_hba-ori.conf 
xcopy /Y "' + $fileAeronConf + '" "' + $pgDataDir + '"
xcopy /Y "' + $filePgHbaTrustConf + '" "' + $pgDataDir + '"
ECHO include_if_exists = ' + "'" + 'aeron.conf' + "'" + ' >> "' + $postgresqlConf + '"

xcopy /Y "' + $scriptBackupDatabase + '" "' + $pgBackupDir + '"
xcopy /Y "' + $scriptCreateDatabase + '" "' + $pgBackupDir + '"
xcopy /Y "' + $scriptRestoreDatabase + '" "' + $pgBackupDir + '"

pause';
$Content | Set-Content -Path $batchCopyPostgresqlConf -Force;


# create_users_aeron_prod_db.bat, run by installer
$Content = '@ECHO OFF

REM Script: Create PostgreSQL users and database ' + $pgDatabase + '
REM Author: John Felix
REM Helpdesk: https://support.broadcastpartners.nl/support/home 
REM Share feedback, suggestions and ideas to support@broadcastpartners.nl

REM Automated installation.


SET HOST=' + $pgHostname + '
SET PORT=' + $pgPort + '
SET DATABASE=' + $pgDatabase + '
SET SCHEMA=' + $pgDBschema + '
SET SUPERUSER=' + $pgSuperUser + '
SET DBADMIN=' + $AeronDBadmin + '

SET PGVERSION=' + $pgVersion + '
SET PGBINDIR=' + $pgBinDir + '
SET PSQL="%PGBINDIR%\psql.exe"

SET DIRSQL=' + $dirSql + '


ECHO Connect to host %HOST% at port %PORT%

ECHO Create users. Enter password for user %SUPERUSER%...
SET PGPASSWORD=' + $pwdSuperUser +'

%PSQL% -h %HOST% -p %PORT% -U %SUPERUSER% -f "%DIRSQL%\1-create-users.sql"
ECHO Create database %DATABASE%. Enter password for user %SUPERUSER%...
%PSQL% -h %HOST% -p %PORT% -U %SUPERUSER% -f "%DIRSQL%\2-create-%DATABASE%.sql"

ECHO Set connect options. Enter password for user %DBADMIN%...
SET PGPASSWORD=' + $pwdAeronDBadmin + '

%PSQL% -h %HOST% -p %PORT% -U %DBADMIN% -d %DATABASE% -f "%DIRSQL%\3-set-connect-options-%DATABASE%.sql"'; 

if ($pgDBschema -ne 'public') {
$Content = $Content + 
'

ECHO Create schema. Enter password for user %DBADMIN%...
%PSQL% -h %HOST% -p %PORT% -U %DBADMIN% -d %DATABASE% -f "%DIRSQL%\4-create-schema-%DATABASE%.sql"';
}
$Content = $Content +
'

ECHO.
ECHO STEP 6
ECHO. 
ECHO Create users and database.
ECHO.

pause';
$Content | Set-Content -Path $batchCreateUsersDatabase -Force;


# copy_postgresql_Hba.bat, run by installer
$Content = '@ECHO OFF

REM Script: Copy PostgreSQL pg_hba.conf
REM Author: John Felix
REM Share feedback, suggestions and ideas to support@broadcastpartners.nl

REM Automated installation - Run script as Administrator

Echo.
Echo Post STEP 6 after create users and database. 
Echo.
Echo Copy pg_hba.conf file.
Echo.

xcopy /Y "' + $filePgHbaConf + '" "' + $pgDataDir + '"
pause';

$Content | Set-Content -Path $batchCopyPostgresqlHba -Force;


# add_firewall_rule_postgresql.bat, run by installer
$Content = '@ECHO OFF

REM Script: Create firewall rule for PostgreSQL server
REM Author: John Felix
REM Helpdesk: https://support.broadcastpartners.nl/support/home 
REM Share feedback, suggestions and ideas to support@broadcastpartners.nl

REM Run script as Administrator

Echo.
Echo STEP 7
Echo.
Echo Add firewall rule
Echo.

netsh advfirewall firewall add rule name="PostgreSQL Database Server" dir=in action=allow protocol=TCP localport='+ $pgPort + '
pause';
$Content | Set-Content -Path $batchAddFirewallRulePostgresql -Force;


# grant_tables_to_aeron_app_role.bat
$Content = '@ECHO OFF

REM -- Run this script after AerOn database conversion or after AerOn first run as user ' + $AeronDBadmin + '.

REM Script: Grant tables to aeron_app_role.
REM Author: John Felix
REM Helpdesk: https://support.broadcastpartners.nl/support/home 
REM Share feedback, suggestions and ideas to support@broadcastpartners.nl


SET HOST=' + $pgHostname + '
SET PORT=' + $pgPort + '
SET DATABASE=' + $pgDatabase + '
SET DBADMIN=' + $AeronDBadmin + '

SET PGVERSION=' + $pgVersion + '
SET PGBINDIR=' + $pgBinDir + '
SET PSQL="%PGBINDIR%\psql.exe"

SET DIRSQL=' + $dirSql + '


ECHO Connect to host %HOST% at port %PORT%

ECHO Grant tables to aeron_app_role. Enter password for user %DBADMIN%...
SET PGPASSWORD=' + $pwdAeronDBadmin +'

%PSQL% -h %HOST% -p %PORT% -U %DBADMIN% -d %DATABASE% -f "%DIRSQL%\5-grant-tables-to-aeron-app-role.sql"

ECHO. 
ECHO Grant tables to ' + $AeronAppUser + '.
ECHO.

pause';
$Content | Set-Content -Path $batchGrantTablesToAeronAppRole -Force;


# Restart PostgreSQL.bat, run by installer
$Content = '@ECHO OFF

REM Script: Restart PostgreSQL service.
REM Author: John Felix
REM Helpdesk: https://support.broadcastpartners.nl/support/home 
REM Share feedback, suggestions and ideas to support@broadcastpartners.nl

REM Run this script as Administrator

ECHO.
ECHO Restart PostgreSQL service...
ECHO.

Net stop ' + $pgServiceName + '
Net start ' + $pgServiceName

$Content | Set-Content -Path $batchRetartPostgresql -Force;

# >> installerscripts
}

function PsqlScripts
{
# Script - reload-postgresql-conf.bat
$Content = '@ECHO OFF

REM Script: Reload PostgreSQL configuration
REM Author: John Felix
REM Helpdesk: https://support.broadcastpartners.nl/support/home 
REM Share feedback, suggestions and ideas to support@broadcastpartners.nl


SET HOST=127.0.0.1
SET PORT=' + $pgPort + '
SET DATABASE=' + $pgDatabase + '
SET SUPERUSER=' + $pgSuperUser + '

SET PGVERSION=' + $pgVersion + '
SET PGBINDIR=' + $pgBinDir + '
SET PSQL="%PGBINDIR%\psql.exe"


ECHO Connect to host %HOST% at port %PORT%.

ECHO Reload PostgreSQL configuration. Enter password for user %SUPERUSER%...
SET PGPASSWORD=' + $pwdSuperUser +'

%PSQL% -c "select pg_reload_conf();"  -h %HOST% -p %PORT% -U %SUPERUSER% -d %DATABASE%

ECHO. 
ECHO Reload PostgreSQL config completed.
ECHO.

pause';
$Content | Set-Content -Path $scriptReloadPsqlConfiguration -Force;

# Script - Create_database.bat
$Content = '@ECHO OFF

REM Script: Create Aeron database
REM Author: John Felix
REM Helpdesk: https://support.broadcastpartners.nl/support/home 
REM Share feedback, suggestions and ideas to support@broadcastpartners.nl

SET HOST=127.0.0.1
SET PORT=' + $pgPort + '
SET DATABASE=' + $pgDatabase + '
SET SCHEMA=' + $pgDBschema + '
SET SUPERUSER=' + $pgSuperUser + '
SET DBADMIN=' + $AeronDBadmin + '

SET PGVERSION=' + $pgVersion + '
SET PGBINDIR=' + $pgBinDir + '
SET PSQL="%PGBINDIR%\psql.exe"
SET CREATEDB="%PGBINDIR%\createdb.exe"


ECHO Connect to host %HOST% at port %PORT%

ECHO Create database %DATABASE%.  Enter password for user %SUPERUSER%...
SET PGPASSWORD=' + $pwdSuperUser +'

%CREATEDB% -h %HOST% -p %PORT% -U %SUPERUSER% -E UTF8 -O %DBADMIN% %DATABASE%

SET PGPASSWORD=' + $pwdAeronDBadmin;

if ($pgDBschema -ne 'public') {
$Content = $Content + 
'

ECHO Create schema. Enter password for user %DBADMIN%...
%PSQL% -c "CREATE SCHEMA %SCHEMA% AUTHORIZATION %DBADMIN%;" -h %HOST% -p %PORT% -U %DBADMIN% -d %DATABASE%

ECHO Set search_path. Enter password for user %DBADMIN%...
%PSQL% -c "alter database %DATABASE% set search_path = ' + "'" + '"$users"' + "'"  +  ', %SCHEMA%, public;"  -h %HOST% -p %PORT% -U %DBADMIN% -d %DATABASE%
ECHO Revoke connect from public. Enter password for user %DBADMIN%...
%PSQL% -c "revoke connect on database %DATABASE% from public;" -h %HOST% -p %PORT% -U %DBADMIN% -d %DATABASE%
';
}

$Content = $Content + 
'

ECHO Grant connect to aeron_app_role. Enter password for user %DBADMIN%...
%PSQL% -c "grant connect on database %DATABASE% to aeron_app_role;" -h %HOST% -p %PORT% -U %DBADMIN% -d %DATABASE%
ECHO Grant usage of schema %SCHEMA% to aeron_app_role. Enter password for user %DBADMIN%...
%PSQL% -c "grant usage on schema %SCHEMA% to aeron_app_role;" -h %HOST% -p %PORT% -U %DBADMIN% -d %DATABASE%
ECHO Grant tables to aeron_app_role. Enter password for user %DBADMIN%...
%PSQL% -c "grant all on all tables in schema %SCHEMA% to aeron_app_role;" -h %HOST% -p %PORT% -U %DBADMIN% -d %DATABASE%

ECHO. 
ECHO Create database completed.
ECHO.

pause';
$Content | Set-Content -Path $scriptCreateDatabase -Force;

# Script - Backup_database.bat
$Content = '@ECHO OFF

REM Script: Backup Aeron database
REM Author: John Felix
REM Helpdesk: https://support.broadcastpartners.nl/support/home 
REM Share feedback, suggestions and ideas to support@broadcastpartners.nl

for /f %%i in (' + "'" + 'powershell ^(get-date^).DayOfWeek' + "'" + ') do SET WEEKDAY=%%i

SET SCRIPTDIR=%~dp0

SET HOST=127.0.0.1
SET PORT=' + $pgPort + '
SET DATABASE=' + $pgDatabase + '
SET DBADMIN=' + $AeronDBadmin + '

SET PGVERSION=' + $pgVersion + '
SET PGBINDIR=' + $pgBinDir + '

SET PGDUMP="%PGBINDIR%\pg_dump.exe"


REM #####  SET BACKUP FOLDER  #####
SET BACKUPDIR=' + $pgBackupDir + '

IF NOT EXIST "%BACKUPDIR%" MD "%BACKUPDIR%"

REM #####  SET FILENAME WHITHOUT EXTENSION  #####
SET FILENAME=backup-%PGVERSION%-full-%DATABASE%-%WEEKDAY%


SET PGPASSWORD=' + $pwdAeronDBadmin +'


ECHO Connect to host %HOST% at port %PORT%

REM Compressed backup! - Use pg_restore or pgAdmin to restore database
ECHO Compressed backup database %DATABASE%. Enter password for user %DBADMIN%...
%PGDUMP% -v -Fc -T automation -O -h %HOST% -p %PORT% -U %DBADMIN% -d %DATABASE% -f "%BACKUPDIR%\%FILENAME%.cst"

REM Plain text format - Use psql to restore database
ECHO Full backup database %DATABASE%. Enter password for user %DBADMIN%...
%PGDUMP% -v -T automation -O -h %HOST% -p %PORT% -U %DBADMIN% -d %DATABASE% -f "%BACKUPDIR%\%FILENAME%.sql"


ECHO. 
ECHO File backupped to "%BACKUPDIR%\%FILENAME%.cst" 
ECHO.
ECHO Please check if backup was succesful.
ECHO.

pause';
$Content | Set-Content -Path $scriptBackupDatabase -Force;

# Script - Restore_database.bat
$Content = '@ECHO OFF

REM Script: Restore Aeron database from file
REM Author: John Felix
REM Helpdesk: https://support.broadcastpartners.nl/support/home 
REM Share feedback, suggestions and ideas to support@broadcastpartners.nl

SET SCRIPTDIR=%~dp0

SET HOST=127.0.0.1
SET PORT=' + $pgPort + '
SET DATABASE=' + $pgDatabase + '
SET SCHEMA=' + $pgDBschema + '
SET SUPERUSER=' + $pgSuperUser + '
SET DBADMIN=' + $AeronDBadmin + '

SET PGVERSION=' + $pgVersion + '
SET PGBINDIR=' + $pgBinDir + '

SET DROPDB="%PGBINDIR%\dropdb.exe"
SET CREATEDB="%PGBINDIR%\createdb.exe"
SET PSQL="%PGBINDIR%\psql.exe"
SET PGRESTORE="%PGBINDIR%\pg_restore.exe"


REM #####  SET SOURCE FOLDER  #####
SET SOURCEDIR=' + $pgBackupDir + '


REM #####  SET FILENAME WHITHOUT EXTENSION  #####
SET FILENAME=backup-' + $pgVersion + '-full-' + $pgDatabase + '


IF EXIST "%SOURCEDIR%\%FILENAME%.sql" (
  GOTO RESTORE
)
IF EXIST "%SOURCEDIR%\%FILENAME%.cst" (
  GOTO RESTORE	
)

ECHO File not found: %SOURCEDIR%\%FILENAME%.cst
ECHO Press ENTER to EXIT...
PAUSE>nul
EXIT 


:RESTORE

REM SET PGPASSWORD=' + $pwdSuperUser +'

ECHO Connect to host %HOST% at port %PORT%.

ECHO Drop database %DATABASE%.  Enter password for user %SUPERUSER%...
%DROPDB% -h %HOST% -p %PORT% -U %SUPERUSER% %DATABASE%


ECHO Create database %DATABASE%.  Enter password for user %SUPERUSER%...
%CREATEDB% -h %HOST% -p %PORT% -U %SUPERUSER% -E UTF8 -O %DBADMIN% %DATABASE%


SET PGPASSWORD=' + $pwdAeronDBadmin +'

IF EXIST "%SOURCEDIR%\%FILENAME%.sql" (
	ECHO Restore backup to database %DATABASE%. Enter password for user %DBADMIN%...
	%PSQL% -h %HOST% -p %PORT% -U %DBADMIN% -d %DATABASE% -f "%SOURCEDIR%\%FILENAME%.sql"
	GOTO SETDB
)
IF EXIST "%SOURCEDIR%\%FILENAME%.cst" (
	ECHO Restore compressed backup to database %DATABASE%. Enter password for user %DBADMIN%...
	%PGRESTORE% -v -h %HOST% -p %PORT% -U %DBADMIN% -d %DATABASE% "%SOURCEDIR%\%FILENAME%.cst"
	GOTO SETDB
) 


:SETDB

IF %SCHEMA% == aeron (
  ECHO Set search_path. Enter password for user %DBADMIN%...
  %PSQL% -c "alter database %DATABASE% set search_path = ' + "'" + '"$users"' + "'"  +  ', %SCHEMA%, public;"  -h %HOST% -p %PORT% -U %DBADMIN% -d %DATABASE%
  ECHO Revoke connect from public. Enter password for user %DBADMIN%...
  %PSQL% -c "revoke connect on database %DATABASE% from public;" -h %HOST% -p %PORT% -U %DBADMIN% -d %DATABASE%
)

ECHO Grant connect to aeron_app_role. Enter password for user %DBADMIN%...
%PSQL% -c "grant connect on database %DATABASE% to aeron_app_role;" -h %HOST% -p %PORT% -U %DBADMIN% -d %DATABASE%
ECHO Grant usage of schema %SCHEMA% to aeron_app_role. Enter password for user %DBADMIN%...
%PSQL% -c "grant usage on schema %SCHEMA% to aeron_app_role;" -h %HOST% -p %PORT% -U %DBADMIN% -d %DATABASE%
ECHO Grant tables to aeron_app_role. Enter password for user %DBADMIN%...
%PSQL% -c "grant all on all tables in schema %SCHEMA% to aeron_app_role;" -h %HOST% -p %PORT% -U %DBADMIN% -d %DATABASE%


SET PGPASSWORD=' + $pwdSuperUser +'

ECHO.
ECHO VACUUM ANALYZE, optimize and reclaim storage occupied by dead tuples. Enter password for user %SUPERUSER%...
%PSQL% -c "VACUUM VERBOSE ANALYZE;" -h %HOST% -p %PORT% -U %SUPERUSER% -d %DATABASE%
ECHO VACUUM and ANALYZE completed.
ECHO.

ECHO. 
ECHO Restore database completed. Check with pgAdmin if restore was succesful.
ECHO.

pause';
$Content | Set-Content -Path $scriptRestoreDatabase -Force;
# >> PsqlScripts
}


function installsteps
{
# steps-to-install.txt
$Content = 'PostgreSQL on Windows installation

The script creates all batch files needed for installation and also downloads installer binaries for PostgreSQL and AerOn. 
Step 1-7: Step 1 to 7 are automated, please confirm each step with YES. 

Step 8: When automated setup 1-7 is completed, then the NexusDB database to PostgreSQL database conversion program is started.

Step 9-12: When the NexusDB database to PostgreSQL database conversion program is closed then
           the AerOn installer is automatically started. From there follow the guidelines from step 9 to 12.

Automated steps:
1) Start PostgreSQL installer with use of optionfile postgreql_install.opt.

2) When PostgreSQL installation has finished goto the PostgreSQL database folder and rename pg_hba.conf to pg_hba-ori.conf.

3) Copy contents of conf folder and paste into the PostgreSQL database folder.
   - aeron.conf       contains PostgreSQL settings.  Edit this file if you want to change or add custom PostgreSQL settings
   - pg_hba.conf      contains PostgreSQL security access rules.
   - recovery.slave   example file for streaming replication.

4) Add this line at the bottom of Postgresql.conf:  include_if_exists = ''aeron.conf''

5) Restart PostgreSQL service.

6) Run create_users_aeron_prod_db.bat. This script runs several sql statements to create users and database.
  
7) Run script add_firewall_rule_postgresql.bat to allow access from network.

Manual steps:
8) Only for AerOn NexusDB users:  
   Users who migrate from AerOn NexusDB need to convert their database to PostgreSQL. 
   The options needed to connect to the database pop-up in a notepad textfile ConnectOptions.txt.
   For NexusDB database conversion connect to the PostgreSQL database as user ' + $AeronDBadmin + '.
   More information can be found in the Install-Postgresql-Aeron ReadMe.pdf.

   The NexusDB database conversion can run while AerOn NexusDB automation is still Live, this way downtime is minimized.

9) Install AerOn 
   Follow directions in AerOn installer.
   To retain AerOn NexusDB configuration, install AerOn Postgres in a different software folder, .e.g. AeronPostgres, and select a different data folder.

10) Only for AerOn NexusDB users:  Copy AerOn NexusDB settings.  
    If AeronPostgres is installed with different data folder as AerOn NexusDB then copy settings file Aeron.ini from AerOn NexusDB data folder into AerOn Postgres data folder.

11) At first run AerOn Postgres:  
    Connect to PostgreSQL database as user ' + $AeronDBadmin + '. Check if everything works.

12) Grant usage of the PostgreSQL database to aeron_app_role/' + $AeronAppUser + '.
    Run grant_tables_to_aeron_app_role.bat. After running this batch file user ' + $AeronAppUser + ' gets permission to take usage of the database. 
    The user ' + $AeronAppUser + ' can be used for external voicetrackers to connect to the PostgreSQL database.

Note: ' + $AeronAppUser + ' can not modify hourformats. You have to connect Aeron to PostgreSQL database with user ' + $AeronDBadmin + ' then.

';
$Content | Set-Content -Path $fileInstallSteps -Force;
}

function GetDateTime
{
# Get date values  
$Date = Get-Date;  
[String]$Year = $Date.Year;  
[String]$Month = $Date.Month; 
[String]$Day = $Date.Day; 
  
# Pad date parts with leading 0's  
if ( $Month.ToString().Length -lt 2 ) { $Month = '0' + $Month };  
if ( $Day.ToString().Length -lt 2 ) { $Day = '0' + $Day };  

# Output  
$Date = "$Year$Month$Day";  
}

function Get-WebFile ([STRING]$Url, [STRING]$Outfile)
{
  $proxy = [System.Net.WebRequest]::GetSystemWebProxy();
  $proxy.Credentials = [System.Net.CredentialCache]::DefaultCredentials;
  $request = New-Object System.Net.WebCLient;
  $request.UseDefaultCredentials = $true; ## Proxy credentials only
  $request.Proxy.Credentials = $request.Credentials;
  $request.DownloadFile($Url, $Outfile);
}

function CreateShortCutSimple([STRING]$AppLocation, [STRING]$PathLnk, [STRING]$WorkDir, [STRING]$Arguments)
{
  $WshShell = New-Object -ComObject WScript.Shell;
  $Shortcut = $WshShell.CreateShortcut($PathLnk);
  $Shortcut.TargetPath = $AppLocation;
  $Shortcut.Arguments = $Arguments;
  $Shortcut.WorkingDirectory = $WorkDir;
  $Shortcut.Save();
}

function CreateShortcut
{
    [CmdletBinding()]
    param (    
        [parameter(Mandatory=$true)]
        [ValidateScript( {[IO.File]::Exists($_)} )]
        [System.IO.FileInfo] $Target,
    
        [ValidateScript( {[IO.Directory]::Exists($_)} )]
        [System.IO.DirectoryInfo] $OutputDirectory,
    
        [string] $Name,
        [string] $Description,
    
        [string] $Arguments,
        [System.IO.DirectoryInfo] $WorkingDirectory,
    
        [string] $HotKey,
        [int] $WindowStyle = 1,
        [string] $IconLocation,
        [switch] $Elevated
    )
 
    try {
        # region Create Shortcut
        if ($Name) {
            [System.IO.FileInfo] $LinkFileName = [System.IO.Path]::ChangeExtension($Name, "lnk")
        } else {
            [System.IO.FileInfo] $LinkFileName = [System.IO.Path]::ChangeExtension($Target.Name, "lnk")
        }
    
        if ($OutputDirectory) {
            [System.IO.FileInfo] $LinkFile = [IO.Path]::Combine($OutputDirectory, $LinkFileName)
        } else {
            [System.IO.FileInfo] $LinkFile = [IO.Path]::Combine($Target.Directory, $LinkFileName)
        }
 
       
        $wshshell = New-Object -ComObject WScript.Shell
        $shortCut = $wshShell.CreateShortCut($LinkFile) 
        $shortCut.TargetPath = $Target
        $shortCut.WindowStyle = $WindowStyle
        $shortCut.Description = $Description
        $shortCut.WorkingDirectory = $WorkingDirectory
        $shortCut.HotKey = $HotKey
        $shortCut.Arguments = $Arguments
        if ($IconLocation) {
            $shortCut.IconLocation = $IconLocation
        }
        $shortCut.Save()
        #endregion
 
        #region Elevation Flag
        if ($Elevated) {
            $tempFileName = [IO.Path]::GetRandomFileName()
            $tempFile = [IO.FileInfo][IO.Path]::Combine($LinkFile.Directory, $tempFileName)
        
            $writer = new-object System.IO.FileStream $tempFile, ([System.IO.FileMode]::Create)
            $reader = $LinkFile.OpenRead()
        
            while ($reader.Position -lt $reader.Length)
            {        
                $byte = $reader.ReadByte()
                if ($reader.Position -eq 22) {
                    $byte = 34
                }
                $writer.WriteByte($byte)
            }
        
            $reader.Close()
            $writer.Close()
        
            $LinkFile.Delete()
        
            Rename-Item -Path $tempFile -NewName $LinkFile.Name
        }
        #endregion
    } catch {
        Write-Error "Failed to create shortcut. The error was '$_'."
        return $null
    }
    return $LinkFile
}
 
# END REGION Functions


# REGION Installation

# Connect options for AerOn and Database conversion program
$ConnectOptionsTxt = "$ScriptPath\ConnectOptions.txt"
$ConnectOptions = "PostgreSQL database users:

Super user: 
" + $pgSuperUser + "        password = " + $pwdSuperUser + 
"

DB admin:
" + $AeronDBadmin + "       password = " + $pwdAeronDBadmin + 
"

DB user:
" + $AeronAppUser + "  password = " + $pwdAeronAppUser + "

Note: " + $AeronAppUser + " has no permission to alter database. Connect Aeron to PostgreSQL database with user " + $AeronDBadmin + ".


PostgreSQL connect options for AerOn and NexusDB database convert program:

Server / Hostname = " + $pgHostname + "
Port     = " + $pgPort + "
Database = " + $pgDatabase + "
Username = " + $AeronDBadmin + "
Password = " + $pwdAeronDBadmin;
$ConnectOptions | Set-Content -Path $ConnectOptionsTxt -Force;

Write-Host "rootdir = " $ScriptPath;

# Make dirs
if (Test-Path $dirAuto) {
  # // File exists
} else {
  # // File does not exist
  mkdir -Path $dirAuto -Force;
}

if (Test-Path $dirSql) {
  # // File exists
} else {
  # // File does not exist
  mkdir -Path $dirSql -Force;
}

if (Test-Path $dirScripts) {
  # // File exists
} else {
  # // File does not exist
  mkdir -Path $dirScripts -Force;
}

if (Test-Path $dirScriptsPsql) {
  # // File exists
} else {
  # // File does not exist
  mkdir -Path $dirScriptsPsql -Force;
}

if (Test-Path $dirConf) {
  # // File exists
} else {
  # // File does not exist
  mkdir -Path $dirConf -Force;
}

if (Test-Path $dirConfTrust) {
  # // File exists
} else {
  # // File does not exist
  mkdir -Path $dirConfTrust -Force;
}

if (Test-Path $dirCmd) {
  # // File exists
} else {
  # // File does not exist
  mkdir -Path $dirCmd -Force;
}

if (Test-Path $pgBackupDir) {
  # // File exists
} else {
  # // File does not exist
  mkdir -Path $pgBackupDir -Force;
}

# Create installer files and scripts
installsteps;
pgoptionfile;
installerscripts;
PsqlScripts;
sqlfiles;
aeronconf;
pghbaconf;

CreateShortcut -Name $name_copy_postgresql_conf -Target $batchCopyPostgresqlConf -WorkingDirectory $dirCmd -OutputDirectory $ScriptPath -Elevated;
CreateShortcut -Name $name_copy_postgresql_Hba -Target $batchCopyPostgresqlHba -WorkingDirectory $dirCmd -OutputDirectory $ScriptPath -Elevated;
CreateShortcut -Name $name_add_firewall_rule_postgresql -Target $batchAddFirewallRulePostgresql -WorkingDirectory $dirCmd -OutputDirectory $ScriptPath -Elevated;
CreateShortcut -Name $name_restart_postgresql -Target $batchRetartPostgresql -WorkingDirectory $dirAuto -OutputDirectory $ScriptPath -Elevated;
CreateShortcut -Name $name_reload_postgresql_conf -Target $scriptReloadPsqlConfiguration -WorkingDirectory $dirScriptsPsql -OutputDirectory $ScriptPath;
CreateShortcut -Name $name_grant_tables_to_aeron_app_role -Target $batchGrantTablesToAeronAppRole -WorkingDirectory $dirAuto -OutputDirectory $ScriptPath;


if (Test-Path $fileInstallSteps) {
  # // File exists
} else {
  # // File does not exist
  Write-Host $ScriptPath;
  Write-Host "Files could not be created.  Please run script from other folder...";
  Write-Host "Press any key to exit script...
  ";
  [System.Reflection.Assembly]::LoadWithPartialName("System.Windows.Forms");
  [System.Windows.Forms.MessageBox]::Show("Files could be created.  Run script from other folder...." , "Folder");
  [void][System.Console]::ReadKey($true);
  exit;
}

try {
  Start-Process notepad -ArgumentList $fileInstallSteps;
} catch {
  Write-Error "Could not open " $fileInstallSteps $_.Exception.Message;
  Write-Host "Exit script.";
  break;
}

try {
  # Download Aeron manual Dutch
  if (Test-Path $aeronHandleidingPath) {
    # // File exists
    Write-Host "
AerOn manual Dutch present.
Download updated manual.
    ";
    Remove-Item $aeronHandleidingPath
    Invoke-WebRequest $aeronHandleidingUrl -OutFile $aeronHandleidingPath;
  } else {
	  
   # // File does not exist
    Write-Host "
Download AerOn manual Dutch...    
    ";
    # Get-WebFile -Url $aeronHandleidingUrl -Outfile $aeronHandleidingPath;   # PowerShell 1.0 / 2.0
    Invoke-WebRequest $aeronHandleidingUrl -OutFile $aeronHandleidingPath; # PowerShell 3.0 and higer
  }
} catch {
  Write-Error "Could not download AerOn manual Dutch" $_.Exception.Message;
  # break;
}
if (Test-Path $aeronHandleidingPath) {
  # // File exists
  Start-Process -FilePath $aeronHandleidingPath -WorkingDirectory $ScriptPath;
}

try {
  # Download Aeron manual English
  if (Test-Path $aeronManualPath) {
    # // File exists
    Write-Host "
AerOn manual EN is present.
Download updated manual.
    ";
    Remove-Item $aeronManualPath
    Invoke-WebRequest $aeronManualUrl -OutFile $aeronManualPath;
  } else {
    # // File does not exist
    Write-Host "
Download AerOn manual English...    
    ";
    # Get-WebFile -Url $aeronHandleidingUrl -Outfile $aeronHandleidingPath;   # PowerShell 1.0 / 2.0
    Invoke-WebRequest $aeronManualUrl -OutFile $aeronManualPath; # PowerShell 3.0 and higer
  }
} catch {
  Write-Error "Could not download AerOn manual English" $_.Exception.Message;
  # break;
}

if (Test-Path $aeronManualPath) {
  # // File exists
  Start-Process -FilePath $aeronManualPath -WorkingDirectory $ScriptPath;
}

try {
  # Download PostgreSQL installer
  if (Test-Path $pgInstallerPath) {
    # // File exists
    Write-Host "PostgreSQL installer is already present, no need to download.
    "
  } else {
    # // File does not exist
    Write-Host "Download PostgreSQL installer...
    ";
    # Get-WebFile -Url $pgInstallerUrl -Outfile $pgInstallerPath; # PowerShell 1.0 / 2.0
    Invoke-WebRequest $pgInstallerUrl -OutFile $pgInstallerPath; # PowerShell 3.0 and higer

    $Arguments = " --optionfile " + '"'+ $filePgInstallerOption + '"'
    CreateShortCutSimple -AppLocation $pgInstallerPath -PathLnk $pgInstallerLnk -WorkDir $ScriptPath -Arguments $Arguments
  }
} catch {
  Write-Error "Could not download PostgreSQL installer " $_.Exception.Message;
  Write-Host "Exit script.";
  break;
}

try {
  # Download NexusDB to PostgreSQL AerOn database conversion program
  if (Test-Path $aeronDbConvertPath) {
    # // File exists
    Write-Host "NexusDB to PostgreSQL database conversion program already present.
    ";
  } else {
    # // File does not exist
    Write-Host "Download NexusDB to PostgreSQL database conversion program...
    ";
    # Get-WebFile -Url $aeronDbConvertUrl -Outfile $aeronDbConvertPath;   # PowerShell 1.0 / 2.0
    Invoke-WebRequest $aeronDbConvertUrl -OutFile $aeronDbConvertPath; # PowerShell 3.0 and higer
  }
} catch {
  Write-Error "Could not download NexusDB to PostgreSQL database conversion program " $_.Exception.Message;
  #break;
}


try {
  # Download Aeron installer
  if (Test-Path $aeronInstallerPath) {
    # // File exists
    Write-Host "AerOn installer is already present, no need to download.
    ";
  } else {
    # // File does not exist
    Write-Host "Download AerOn installer...
    ";
    # Get-WebFile -Url $aeronInstallerUrl -Outfile $aeronInstallerPath;   # PowerShell 1.0 / 2.0
    Invoke-WebRequest $aeronInstallerUrl -OutFile $aeronInstallerPath; # PowerShell 3.0 and higer
  }
} catch {
  Write-Error "Could not download AerOn installer " $_.Exception.Message;
  Write-Host "Exit script.";
  break;
}

# Start installation or exit
if ($OnlyDownload) {
  Write-Host "Only download activated.";
  Write-Host "Press any key to exit script...
  ";
  [void][System.Console]::ReadKey($true);
  exit;
}

try {
  # Run PostgreSQL installer
  if (Test-Path $pgInstallerPath) {
    # // File exists
    Write-Host "Run PostgreSQL installer...
    ";
    $Arguments = " --optionfile " + '"'+ $filePgInstallerOption + '"';
    Start-Process -FilePath $pgInstallerPath -ArgumentList $Arguments -Wait;
    If (Test-Path $pgAdmin4Path)
    {
     CreateShortCutSimple -AppLocation $pgAdmin4Path -PathLnk $pgAdmin4Lnk -WorkDir $pgAdmin4Dir;
    }
  } else {
    # // File does not exist
    Write-Host "File not found...  PostgreSQL installer... Exit script";
    Write-Host "Press any key to continue...
    ";
    [void][System.Console]::ReadKey($true);
    break;
  }
} catch {
  Write-Error "Could not run $pgInstallerPath " $_.Exception.Message;
  break;
}

# Check postgresql.conf
if (Test-Path $postgresqlConf) {
  # // File exists
} else {
  # // File does not exist
  Write-Host "File not found.  Postgresql.conf  ";
  Write-Host $postgresqlConf;
  Write-Host "Press any key to continue...
  ";
  [void][System.Console]::ReadKey($true)
  break;
}

try {
  # Copy PostgreSQL configuration files aeron.conf & pg_hba.conf
  Start-Process -FilePath $lnkCopyPostgresqlConf -Wait;
} catch {
  Write-Error "Could not run $ScriptPath\copy_postgresql_conf.lnk " $_.Exception.Message;
  break;
}

# Check aeron.conf
$aeronConf = $pgDataDir + "\aeron.conf"
if (Test-Path $aeronConf) {
  # // File exists
} else {
  # // File does not exist
  Write-Host "File not found.  aeron.conf  ";
  Write-Host $aeronConf;
  Write-Host "Press any key to continue...
  ";
  [void][System.Console]::ReadKey($true)
  break;
}

try {
  # Restart PostgresSQL service
  Write-Host "Restart PostgreSQL service...
  ";
  Start-Process -FilePath $lnkRestartPostgresql -Wait;
} catch {
  Write-Error "Could not run $batchRetartPostgresql " $_.Exception.Message;
  break;
}

try {
  # Create PostgreSQL database and users
  Write-Host "Create PostgreSQL database and users...
  ";

  Start-Process -FilePath $batchCreateUsersDatabase -WorkingDirectory $ScriptPath -Wait;
} catch {
  Write-Error "Could not run $batchCreateUsersDatabase " $_.Exception.Message;
  break;
}

try {
  # Copy pg_hba.conf
  Start-Process $lnkCopyPostgresqlHba -Wait;
} catch {
  Write-Error "Could not run $batchCopyPostgresqlHba " $_.Exception.Message;
  break;
}

try {
  # Restart PostgresSQL service
  Write-Host "Restart PostgreSQL service...
  ";
  Start-Process -FilePath $lnkRestartPostgresql -Wait;
} catch {
  Write-Error "Could not run $batchRetartPostgresql " $_.Exception.Message;
  break;
}

try {
  # Add firewall rule for PostgreSQL
  Write-Host "Add Firewall rule for PostgreSQL server...
  ";
  Start-Process -FilePath $lnkAddFirewallRulePostgresql;
} catch {
  Write-Error "Could not run $batchAddFirewallRulePostgresql " $_.Exception.Message;
  break;
}

try {
  # Show Connnect options
  Write-Host $ConnectOptions;
  Start-Process notepad -ArgumentList $ConnectOptionsTxt;
} catch {
  Write-Error "Could not open $ConnectOptionsTxt " $_.Exception.Message;
  break;
}

if ($RunDbConvert) {
  try {
    # Run AerOn database conversion program
    if (Test-Path $aeronDbConvertPath) {
      # // File exists
      Write-Host "Start AerOn database conversion program...
      ";
      Start-Process -FilePath $aeronDbConvertPath -Wait;
    } else {
      # // File does not exist
      Write-Host "File not found.  AerOn database conversion program.  Exit script.";
      Write-Host "Press any key to continue...
      ";
      [void][System.Console]::ReadKey($true);
      exit;
    }
  } catch {
    Write-Error "Could not run $aeronDbConvertPath " $_.Exception.Message;
    break;
  }
}

try {
  # Run AerOn installer
  if (Test-Path $aeronInstallerPath) {
    # // File exists
    Write-Host "Run AerOn installer...
    ";
    Start-Process -FilePath $aeronInstallerPath -Wait;
  } else {
    # // File does not exist
    Write-Host "File not found.  AerOn installer. Exit script.";
    Write-Host "Press any key to continue...
    ";
    [void][System.Console]::ReadKey($true);
    exit;
  }
} catch {
  Write-Error "Could not run $aeronInstallerPath " $_.Exception.Message;
  break;
}

# !!! Done !!!
Write-Host "!!! Installation complete !!!
";
Write-Host "
Press any key to continue...";
[void][System.Console]::ReadKey($true);

# END REGION  Installation
#>